/* SofChat (Web Chat program SHM version)      */
/* Copyright(c) 1996-2000 Y.YONEYAMA           */
/*                                             */
/* export.c                                    */

#include <stdio.h>
#include <fcntl.h>
#include <time.h>

#include "export.h"
#include "code.h"

#define MAX_SAVE_USERS  1000

static int expf = -1;

int export_open(fname)
char *fname;
{
  if(expf != -1)
    export_close();
  if(*fname == '\0') return(-1);
  expf = open(fname, O_TRUNC|O_WRONLY, 0600);
  if(expf == -1) return(-1);
  return(expf);
}

static int export_mopen(fname, bsize)
char *fname;
int bsize;
{
  int len;
  if(expf != -1)
    export_close();
  if(*fname == '\0') return(-1);
  expf = open(fname, O_RDWR, 0644);
  if(expf == -1) return(-1);
  len = lseek(expf, 0, SEEK_END);
  lseek(expf, 0, SEEK_SET);
  return(len / bsize);
}

int export_close()
{
  if(expf != -1)
    close(expf);
  expf = -1;
  return(0);
}

int export_user(fname, user, rename)
char *fname;
USERINFO *user;
char *rename;
{
  char wbuf[21+25+81+21+21+11+11+11+11+11];
  int len = 0, i, u;
  if((u = export_mopen(fname, sizeof(wbuf))) != -1)
  {
    char name[21], cpasswd[25];
    for(i=0;i<u;i++) /* search user entry */
    {
      read(expf, wbuf, sizeof(wbuf));
      strncpy(name, &wbuf[0], 21);
      if(strcmp(name, user->name) == 0) break;
    }
    if(i >= MAX_SAVE_USERS)
    {
      export_close(expf);
      return(-1);
    }
    lseek(expf, i*sizeof(wbuf), SEEK_SET);
    memset(wbuf, 0, sizeof(wbuf));
    memset(cpasswd, 0, sizeof(cpasswd));
    encode(cpasswd, user->password);
    if(rename != 0)
      strncpy(&wbuf[0], rename, 21);
    else
      strncpy(&wbuf[0], user->name, 21);
    strncpy(&wbuf[21], cpasswd, 25);
    strncpy(&wbuf[21+25], user->address, 81);
    strncpy(&wbuf[21+25+81], user->namecolor, 21);
    strncpy(&wbuf[21+25+81+21], user->textcolor, 21);
    sprintf(&wbuf[21+25+81+21+21], "%010d", user->refresh);
    sprintf(&wbuf[21+25+81+21+21+11], "%08d", user->maxlines);
    sprintf(&wbuf[21+25+81+21+21+11+9], "%01d", user->permit);
    sprintf(&wbuf[21+25+81+21+21+11+11], "%010d", user->count);
    sprintf(&wbuf[21+25+81+21+21+11+11+11], "%010d", (int)user->btime);
    sprintf(&wbuf[21+25+81+21+21+11+11+11+11], "%010d", user->flags);
    len = write(expf, wbuf, sizeof(wbuf));
    export_close(expf);
    return(len);
  }
  return(-1);
}

int export_text(name, message, mdate)
char *name;
char *message;
time_t mdate;
{
  char wbuf[21+1025+11];
  int len = 0;
  memset(wbuf, 0, sizeof(wbuf));
  strncpy(&wbuf[0], name, 21);
  wbuf[20] = '\0';
  strncpy(&wbuf[21], message, 1024);
  wbuf[21+1024] = message[1024];
  sprintf(&wbuf[21+1025], "%010d", mdate);
  if(expf != -1)
    len = write(expf, wbuf, sizeof(wbuf));
  return(len);
}

int import_open(fname)
char *fname;
{
  if(expf != -1)
    export_close();
  expf = open(fname, O_RDONLY, 0644);
  return(expf);
}

int import_close()
{
  return(export_close());
}

int import_user(user)
USERINFO *user;
{
  char wbuf[21+25+81+21+21+11+11+11+11+11];
  int len = 0;
  if(expf != -1)
  {
    len = read(expf, wbuf, sizeof(wbuf));
    if(len == sizeof(wbuf))
    {
      char cpasswd[25];
      memset(cpasswd, 0, sizeof(cpasswd));
      strncpy(user->name, &wbuf[0], 21);
      strncpy(cpasswd, &wbuf[21], 25);
      decode(user->password, cpasswd);
      strncpy(user->address, &wbuf[21+25], 81);
      strncpy(user->namecolor, &wbuf[21+25+81], 21);
      strncpy(user->textcolor, &wbuf[21+25+81+21], 21);
      sscanf(&wbuf[21+25+81+21+21], "%010d", &user->refresh);
      sscanf(&wbuf[21+25+81+21+21+11], "%08d", &user->maxlines);
      sscanf(&wbuf[21+25+81+21+21+11+9], "%01d", &user->permit);
      sscanf(&wbuf[21+25+81+21+21+11+11], "%010d", &user->count);
      sscanf(&wbuf[21+25+81+21+21+11+11+11], "%010d", &user->btime);
      sscanf(&wbuf[21+25+81+21+21+11+11+11+11], "%010d", &user->flags);
    }
    else
      return(0);
  }
  return(len);
}

int read_user(fname, user, name, password)
char *fname;
USERINFO *user;
char *name;
char *password;
{
  char wbuf[21+25+81+21+21+11+11+11+11+11];
  int len = 0;
  import_open(fname);
  while(expf != -1)
  {
    len = read(expf, wbuf, sizeof(wbuf));
    if(len == sizeof(wbuf))
    {
      char tmpname[21];
      char tmppassword[21];
      char cpasswd[25];
      strncpy(tmpname, &wbuf[0], 21);
      memset(cpasswd, 0, sizeof(cpasswd));
      strncpy(cpasswd, &wbuf[21], 25);
      decode(tmppassword, cpasswd);
      if(((*name != '\0') && (strcmp(tmpname, name) == 0)) || ((*name == '\0') && strcmp(tmppassword, password)))
      {
        strncpy(user->name, tmpname, 21);
        strncpy(user->password, tmppassword, 11);
        strncpy(user->address, &wbuf[21+25], 81);
        strncpy(user->namecolor, &wbuf[21+25+81], 21);
        strncpy(user->textcolor, &wbuf[21+25+81+21], 21);
        sscanf(&wbuf[21+25+81+21+21], "%010d", &user->refresh);
        sscanf(&wbuf[21+25+81+21+21+11], "%08d", &user->maxlines);
        sscanf(&wbuf[21+25+81+21+21+11+9], "%01d", &user->permit);
        sscanf(&wbuf[21+25+81+21+21+11+11], "%010d", &user->count);
        sscanf(&wbuf[21+25+81+21+21+11+11+11], "%010d", &user->btime);
        sscanf(&wbuf[21+25+81+21+21+11+11+11+11], "%010d", &user->flags);
        break;
      }
    }
    else
    {
      len = 0;
      break;
    }
  }
  import_close();
  return(len);
}

int import_text(name, message, mdate)
char *name;
char *message;
time_t *mdate;
{
  char wbuf[21+1025+11];
  int len = 0;
  if(expf != -1)
  {
    len = read(expf, wbuf, sizeof(wbuf));
    if(len == sizeof(wbuf))
    {
      strncpy(name, &wbuf[0], 21);
      name[40] = '\0';
      strncpy(message, &wbuf[21], 1024);
      message[1024] = wbuf[21+1024];
      sscanf(&wbuf[21+1025], "%010d", mdate);
    }
    else
      return(0);
  }
  return(len);
}

